@@ -13,7 +13,7 @@ use HTML::Blitz::TokenType qw(
     TT_DOCTYPE
 );
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 method _fail(
     $msg,
@@ -109,6 +109,10 @@ my %void_tags = map +($_ => 1), qw(
     source
     track
     wbr
+
+    basefont bgsound
+    frame
+    keygen
 );
 
 my %foreign_tags = map +($_ => 1), qw(
