@@ -4,7 +4,7 @@
 # See the "COPYING" file for details.
 package HTML::Blitz::pragma;
 use strict;
-use warnings;
+use warnings qw(all FATAL uninitialized);
 use constant PERL_VERSION => '5.20';
 use feature ':' . PERL_VERSION;
 use Function::Parameters 2;
@@ -12,7 +12,7 @@ no indirect ':fatal';
 
 use Carp ();
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 method import($class: @items) {
     for my $item (@items) {
@@ -20,7 +20,7 @@ method import($class: @items) {
     }
 
     strict->import;
-    warnings->import;
+    warnings->import(qw(all FATAL uninitialized));
     feature->import(':' . PERL_VERSION);
     Function::Parameters->import;
     indirect->unimport(':fatal');
